//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "LivingText0.h"
#include <math.h>
#include <stdio.h>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

#define Deg2Rad  3.141592653/180.

// Don't name "Left" because some components use that !!!
enum TextPosition { LEFTUP=1,   UP,     RIGHTUP,
                    LEFT,       CENTER, RIGHT,
                    LEFTBOTTOM, BOTTOM, RIGHTBOTTOM,
                    USERDEFINE };

void putText(TCanvas *canvas, int x, int y, char *text,
             TextPosition tp=LEFTUP, int xc=0, int yc=0)
{ int h, w;
  TBrushStyle oldBrushStyle;
  h=canvas->TextHeight(text);
  w=canvas->TextWidth(text);
  switch(tp)
    { case      LEFTUP : x-=w;      y-=h;      break;
      case          UP : x-=(w>>1); y-=h;      break;
      case     RIGHTUP :            y-=h;      break;
      case        LEFT : x-=w;      y-=(h>>1); break;
      case      CENTER : x-=(w>>1); y-=(h>>1); break;
      case       RIGHT :            y-=(h>>1); break;
      case  LEFTBOTTOM : x-=w;                 break;
      case      BOTTOM : x-=(w>>1);            break;
      case RIGHTBOTTOM : break;
      case  USERDEFINE : x-=xc;     y-=yc;
    };
  oldBrushStyle=canvas->Brush->Style;
  canvas->Brush->Style=bsClear;  //
  canvas->TextOut(x, y, text);
  canvas->Brush->Style=oldBrushStyle;

}
// (xc, yc) : the center of rotating text
// (x0, y0) : the relative position to (xc, yc), also assigned by tp.
// It's available for truetype fonts only !!!
void rotateText(TCanvas *canvas, int xc, int yc, char *text,
                TextPosition tp=LEFTUP, int angle=0, int x0=0, int y0=0)
{ int h, w, xn, yn;
  float cosine, sine;
  TLogFont lf;
  TBrushStyle oldBrushStyle;

  GetObject(canvas->Font->Handle, sizeof(TLogFont), &lf);

  h=canvas->TextHeight(text);
  w=canvas->TextWidth(text);
  switch(tp)
    { case      LEFTUP : x0=xc;        y0=yc;        break;
      case          UP : x0=xc-(w>>1); y0=yc;        break;
      case     RIGHTUP : x0=xc-w;      y0=yc;        break;
      case        LEFT : x0=xc;        y0=yc-(h>>1); break;
      case      CENTER : x0=xc-(w>>1); y0=yc-(h>>1); break;
      case       RIGHT : x0=xc-w;      y0=yc-(h>>1); break;
      case  LEFTBOTTOM : x0=xc;        y0=yc-h;      break;
      case      BOTTOM : x0=xc-(w>>1); y0=yc-h;      break;
      case RIGHTBOTTOM : x0=xc-w;      y0=yc-h;      break;
      case  USERDEFINE : x0=xc-x0;     y0=yc-y0;     break; //case of user defines (xc, yc)
    };
  oldBrushStyle=canvas->Brush->Style;
  canvas->Brush->Style=bsClear;  //
  cosine=cos(angle*Deg2Rad);  sine=sin(angle*Deg2Rad);
  xn= xc + (x0-xc)*cosine + (y0-yc)*sine;
  yn= yc - (x0-xc)*sine   + (y0-yc)*cosine;

  lf.lfEscapement=(angle)*10;
  /*
  lf.lfHeight=h;
  lf.lfWidth =0;
  lf.lfWeight = FW_NORMAL;
  lf.lfCharSet = DEFAULT_CHARSET;
  lf.lfOutPrecision = OUT_DEFAULT_PRECIS;
  lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
  lf.lfQuality = DEFAULT_QUALITY;
  lf.lfPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;
  */
  canvas->Font->Handle=CreateFontIndirect(&lf);
  canvas->TextOut(xn, yn, text);

  canvas->Brush->Style=oldBrushStyle;
}
class FallingText
{ protected:
    float ma;
    float mDegree;
    float mt;
    bool mFalling;
    AnsiString mText;
    int mX, mY;
  public:
    FallingText() {}
    FallingText(char *text, int x0, int y0, float a=1, float deg=0 )
      { mText = (AnsiString)"" + text;
        ma = a;
        mDegree = deg;
        mt = 0.0;
        mX = x0;
        mY = y0;
        mFalling = false;
      }
    FallingText(AnsiString text, int x0, int y0, float a=1, float deg=0)
      { mText = text;
        ma = a;
        mDegree = deg;
        mt = 0.0;
        mX = x0;
        mY = y0;
        mFalling = false;
      }
    void setting(char *text, int x0, int y0, float a=1, float deg=0 )
      { mText = (AnsiString)"" + text;
        ma = a;
        mDegree = deg;
        mt = 0.0;
        mX = x0;
        mY = y0;
        mFalling = false;
      }
    void setA(float a) { ma=a; }
    void setDegree(float degree) { mDegree=degree; }
    void setText(char *text) { mText = (AnsiString)"" + text; }
    void setText(AnsiString text) { mText = text; }
    void moveTo(int nx, int ny) { mX=nx; mY=ny; }
    void move(int dx, int dy)  { mX+=dx; mY+=dy; }
    void show(TCanvas *canvas, TColor color)
      { rotateText(canvas, mX, mY, mText.c_str(), CENTER, mDegree);
      }
    void start() { mFalling = true; }
    bool isFalling() { return mFalling; }
    void next(float dt=1, float deg=0)
      { if ( ! mFalling ) return;
        mY += 0.5*ma*(2*mt-dt)*dt;
        mt += dt;
        mDegree += deg;
      }
};
class MovingText
{ protected:
    float mVx, mVy;
    float mDegree;
    bool  mMoving;
    AnsiString mText;
    int mX, mY;
  public:
    MovingText(char *text, int x0, int y0, float vx=0, float vy=0, float deg=0 )
      { mText = (AnsiString)"" + text;
        mVx = vx;
        mVy = vy;
        mDegree = deg;
        mX = x0;
        mY = y0;
        mMoving = false;
      }
    MovingText(AnsiString text, int x0, int y0, float vx=0, float vy=0, float deg=0 )
      { mText = (AnsiString)"" + text;
        mVx = vx;
        mVy = vy;
        mDegree = deg;
        mX = x0;
        mY = y0;
        mMoving = false;
      }
    void  setVelocity(float vx, float vy) { mVx=vx; mVy=vy; }
    void  setDegree(float degree) { mDegree=degree; }
    void  setText(char *text) { mText = (AnsiString)"" + text; }
    void  setText(AnsiString text) { mText = text; }
    void  moveTo(int nx, int ny) { mX=nx; mY=ny; }
    void  move(int dx, int dy)  { mX+=dx; mY+=dy; }
    void  start() { mMoving=true; }
    bool  isMoving () { return mMoving; }
    float toDegree()
      { float theta;
        theta = atan(mVy/mVx)*180/3.141592653;
        if ( mVx<0.0 && mVy>0 ) theta+=180;
        else theta-=180;
        return(theta);
      }
    void show(TCanvas *canvas, TColor color)
      { rotateText(canvas, mX, mY, mText.c_str(), CENTER, mDegree);
      }
    void next(float dt=1, float deg=0)
      { if ( !mMoving ) return;
        mX += mVx*dt;
        mY -= mVy*dt;
        mDegree += deg;
      }
};
struct Position
{ int x, y;
};
class Banner
{ protected:
    Position *mPos;
    int mLength;
    char *mWord;
  public:
    ~Banner() { if ( mPos!=NULL ) delete [] mPos; }
    Banner()  { mPos=NULL; mLength=0; }
    void setText(char *text) { mWord=text; }
    void getBanner(TCanvas *canvas, int x=0, int y=0)
      { int height, width, index;

        width = canvas->TextWidth(mWord);
        height= canvas->TextHeight(mWord);
        if ( mPos!=NULL ) delete [] mPos;
        mPos = new Position[width*height];
        canvas->Font->Color = clRed;
        canvas->TextOut(0, 0, mWord);
        index = 0;
        for (int r=0; r<height; r++)
           { for (int c=0; c<width; c++)
                { if ( canvas->Pixels[c+x][r+y]!=(TColor)RGB(255,255,255) )  //canvas->Font->Color )
                    { mPos[index  ].x = c+x;
                      mPos[index++].y = r+y;
                    }
                }
           }
        mLength = index;
        //ShowMessage(IntToStr(mLength));
      }
    void show(TCanvas * canvas, int x, int y, float scale=1.0, char *ch=NULL)
      { int height, width, index;

        if ( ch==NULL ) ch=mWord;
        width = canvas->TextWidth(ch) * scale;
        height= canvas->TextHeight(ch)* scale;
        for (index=0; index<mLength; index++)
           { putText(canvas, x+width*mPos[index].x, y+height*mPos[index].y,
                     ch, CENTER);
           }
      }
    void rotatingShow(TCanvas * canvas, int x, int y, float scale=1.0, char *ch=NULL)
      { int height, width, index;

        if ( ch==NULL ) ch=mWord;
        width = canvas->TextWidth(ch) * scale;
        height= canvas->TextHeight(ch)* scale;
        for (int deg=0; deg<=360; deg+=10){
        Form1->Image1->Canvas->Rectangle(0,0,300,300);
        for (index=0; index<mLength; index++)
           {
             //rotateText(Form1->Image1->Canvas, x+width*mPos[index].x, y+height*mPos[index].y,
             rotateText(Form1->Image1->Canvas, width*mPos[index].x, height*mPos[index].y,
                        ch, CENTER, deg);

           };
           //Sleep(100);
           canvas->CopyRect(TRect(x,y,x+300,y+300),
                            Form1->Image1->Canvas, TRect(0,0,300,300));
         }
      }
};
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Button1Click(TObject *Sender)
{ int k, r, x0, y0, x, y, deg, scale;
  char msg[10];
  /*
  Form1->Canvas->TextOut(100,100,"TEST");
  Form1->Canvas->MoveTo(100, 100);
  Form1->Canvas->LineTo(200, 100);
  rotateText(Form1->Canvas, 100, 100, "show me", LEFTUP, 90);
  rotateText(Form1->Canvas, 200, 100, "show me", RIGHTUP, 90);
  */
  x0 = y0 = Form1->ClientHeight / 2;
  deg = 30;  r = 100;
  Form1->Canvas->Pen->Color  = clRed;
  Form1->Canvas->Font->Color = clRed;
  Form1->Refresh();
  for (k=1; k<=12; k++)
     { x=x0+r*sin(k*deg*Deg2Rad);
       y=y0-r*cos(k*deg*Deg2Rad);
       sprintf(msg, "%d:text", k);
       rotateText(Form1->Canvas, x, y, msg, RIGHT, 270-k*deg);
       rotateText(Form1->Canvas, x, y, msg, RIGHT,  90-k*deg);
       Form1->Canvas->Pen->Color=clRed;
       Form1->Canvas->MoveTo(x0, y0);
       Form1->Canvas->LineTo(x, y);
     }
  Sleep(1500);

  scale = 200;
  TextPosition index;
  Form1->Refresh();
  for (deg=0; deg<=360; deg+=45)
     { if ( deg<360 )
         Form1->Canvas->Font->Color = clRed;
       else Form1->Canvas->Font->Color = clBlue;

       index = 1;
       for (y=-1; y<=1; y++)
          { for (x=-1; x<=1; x++)
               { rotateText(Form1->Canvas, x0+scale*x, y0+scale*y, "SHOW ME", index, deg);
                 index = index + 1;
               }
          }
       for (y=-1; y<=1; y++)
          { Form1->Canvas->Pen->Color = clBlack;
            Form1->Canvas->MoveTo(x0-scale, y0+scale*y);
            Form1->Canvas->LineTo(x0+scale, y0+scale*y);
            Form1->Canvas->MoveTo(x0+scale*y, y0-scale);
            Form1->Canvas->LineTo(x0+scale*y, y0+scale);
          }
       Sleep(1000);
     }
  /*
  deg=30;
  x0=y0=200; r=100;
  Form1->Canvas->Brush->Style=bsClear;
  Form1->Canvas->Ellipse(x0-100, y0-100, x0+100, y0+100);
  Form1->Canvas->Ellipse(x0-1, y0-1, x0+1, x0+1);
  rotateText(Form1->Canvas, x0, y0, "string", CENTER, 180);
  */
  /*
  
  */
}
//---------------------------------------------------------------------------
TPoint points[4];
int scale=50;

void movePolygon(int x, int y)
{ points[0] = Point(x+     0, y- scale);
  points[1] = Point(x- scale, y-     0);
  points[2] = Point(x+     0, y+ scale);
  points[3] = Point(x+ scale, y-     0);
}

void __fastcall TForm1::BitBtn1Click(TObject *Sender)
{ AnsiString msg;
  int x=70, y=70, h;

  Form1->Refresh();
  ShowMessage(FontDialog1->Font->Name );
  if ( FontDialog1->Execute() )
    { Form1->Canvas->Font = FontDialog1->Font;
      Image1->Canvas->Font= FontDialog1->Font;
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button2Click(TObject *Sender)
{ char * string[]={"A","B","C","D","E","F","G","H","J","I","K","L"};
                   //mnopqrstuvwxyz1234567890";
  ///*
  Form1->Canvas->TextOut(100,100,"TEST");
  rotateText(Form1->Canvas, 100, 100, "A", CENTER, 90);
  Form1->Canvas->MoveTo(100,100);
  Form1->Canvas->LineTo(200,100);
  FallingText ft("", 200, 20, 0, 10);
  FallingText ft1("", 250, 20, 20, 0);

  for (int k=0; k<100; k+=10)
     { ft.next();
       ft.show(Form1->Canvas, clRed);
       ft.setDegree(k*10);
       ft1.next(0.5);
       ft1.show(Form1->Canvas, clRed);
       //ft1.setDegree(k*10);
       Sleep(100);
       Form1->Refresh();
     }
  //*/

  const int total=120;
  FallingText aft[total];
  Form1->Refresh();
  Banner banner0, banner;
  /*
  banner0.setText("");
  banner0.getBanner(Form1->Canvas, 0, 0);
  banner0.show(Form1->Canvas, 0, 0, 0.9, "q");
  banner.setText("");
  banner.getBanner(Form1->Canvas, 0, 0);
  banner.show(Form1->Canvas, 0, 300, 0.9, "");
  */
  //Form1->Canvas->CopyRect(TRect(0, 0,200,200), Form2->Canvas,TRect(0, 0,200,200));
//return;
  ///*
  randomize();
  for (int k=0; k<total; k++)
     { aft[k].setting( string[random(12)],
                       random(300), random(200), 2+random(12), random(30) );
       if ( random(2) ) aft[k].start();
     };

  for (int k=0; k<40; k++)
     { Form1->Refresh();
       banner0.show(Form1->Canvas, 0, 0, 0.5);
       for (int c=0; c<total; c++)
          { if ( aft[k].isFalling() )
              { aft[c].next(0.5);
                aft[c].show(Form1->Canvas, clRed);
              }
            else aft[k].start();
          }
       Sleep(200);
       if ( k==0 ) ShowMessage("Start falling texts!");
     }
  //*/
  banner0.show(Form1->Canvas, 0, 0, 0.5);

  return;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button3Click(TObject *Sender)
{ char * string[]={"A","B","C","D","E","F","G","H","J","I","K","L"};
                   //mnopqrstuvwxyz1234567890";
  const int total=120;
  FallingText aft[total];

  Banner banner0, banner;
  ///*
  //Form1->Canvas->TextOut(100,100,"TEST");
  //rotateText(Form1->Canvas, 100, 100, "A", CENTER, 90);
  //Form1->Canvas->MoveTo(100,100);
  //Form1->Canvas->LineTo(200,100);
  FallingText ft("", 200, 20,  0, 10);
  FallingText ft1("", 250, 20, 20, 0);
  ///*
  for (int k=0; k<1; k+=10)
     { ft.next();
       ft.show(Form1->Canvas, clRed);
       ft.setDegree(k*10);
       ft1.next(0.5);
       ft1.show(Form1->Canvas, clRed);
       //ft1.setDegree(k*10);
       //Sleep(100);
       Form1->Refresh();
     }
  //*/
  //gpBoxFunction->Visible = false;
  Form1->Canvas->TextOut(300, 620, "FnqtBqPqTt wzII");
  //for (int n=0; n<3; n++)
  while (true)
     { banner0.setText("q");
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 0, 0, 0.66, "q");
       banner0.setText("");
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 300, 0, 0.7, "");
       banner0.setText("t");
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 600, 0, 0.66, "t");
       Application->ProcessMessages();
       banner0.setText("q");
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 0, 300, 0.66, "q");
       banner0.setText("q");
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 300, 300, 0.7, "q");
       banner0.setText("t");
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 600, 300, 0.66, "t");
       Form1->Canvas->TextOut(300, 620, "FnqtBqPqTt wzII");
       Application->ProcessMessages();
       if ( !chkRepeat->Checked ) break;
     }
  return;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormResize(TObject *Sender)
{
  gpBoxFunction->Left = ClientWidth - gpBoxFunction->Width -2;
  gpBoxFunction->Top  = ClientHeight- gpBoxFunction->Height-2;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::BitBtn2Click(TObject *Sender)
{ char * string[]={"A","B","C","D","E","F","G","H","J","I","K","L"};
                   //mnopqrstuvwxyz1234567890";
  const int total=120;
  FallingText aft[total];

  Banner banner0, banner;
  ///*
  //Form1->Canvas->TextOut(100,100,"TEST");
  //rotateText(Form1->Canvas, 100, 100, "A", CENTER, 90);
  //Form1->Canvas->MoveTo(100,100);
  //Form1->Canvas->LineTo(200,100);
  FallingText ft("", 200, 20, 0, 10);
  FallingText ft1("", 250, 20, 20, 0);

  for (int k=0; k<1; k+=10)
     { ft.next();
       ft.show(Form1->Canvas, clRed);
       ft.setDegree(k*10);
       ft1.next(0.5);
       ft1.show(Form1->Canvas, clRed);
       //ft1.setDegree(k*10);
       //Sleep(100);
       Form1->Refresh();
     }
  //gpBoxFunction->Visible = false;
  Form1->Canvas->TextOut(300, 620, "FnqtBqPqTt wzII");
  //for (int n=0; n<3; n++)
  while ( true )
     { banner0.setText("");
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 0, 0, 0.66, "");
       banner0.setText("`");
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 300, 0, 0.7, "`");
       banner0.setText("");
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 600, 0, 0.67, "");
       Application->ProcessMessages();
       banner0.setText("z");
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 150, 300, 0.7, "z");
       banner0.setText("n");
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 450, 300, 0.7, "n");
       Form1->Canvas->TextOut(300, 620, "FnqtBqPqTt wzII");
       Application->ProcessMessages();
       if ( !chkRepeat->Checked ) break;
     }
  return;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormMouseDown(TObject *Sender, TMouseButton Button,
      TShiftState Shift, int X, int Y)
{
  if ( Button==mbRight )
    { gpBoxFunction->Visible = true;
    }
}
//---------------------------------------------------------------------------
// code segment
/*
  Form1->Canvas->Pen->Color = clRed;
  Form1->Canvas->Brush->Color = clRed;
  movePolygon(x,y);
  Form1->Canvas->Polygon(points, 3);
  msg = "K"; //"";
  Form1->Canvas->Pen->Color = clBlack;
  rotateText(Form1->Canvas, x, y, msg.c_str(), CENTER, 180);
  Form1->Canvas->Pen->Color = clRed;
  Form1->Canvas->Brush->Color = clRed;
  movePolygon(x,y+111);
  Form1->Canvas->Polygon(points, 3);
  msg = "";
  Form1->Canvas->Pen->Color = clBlack;
  rotateText(Form1->Canvas, x, y+111, msg.c_str(), CENTER, 180);
  */
void __fastcall TForm1::bbtnNewClick(TObject *Sender)
{ char title[11];
  char msg[3];
  const int total=120;
  FallingText aft[total];

  Banner banner0;
  ///*
  FallingText ft("", 200, 20, 0, 10);
  FallingText ft1("", 250, 20, 20, 0);

  for (int k=0; k<0; k+=10)
     { ft.next();
       ft.show(Form1->Canvas, clRed);
       ft.setDegree(k*10);
       ft1.next(0.5);
       ft1.show(Form1->Canvas, clRed);
       //ft1.setDegree(k*10);
       //Sleep(100);
       Form1->Refresh();
     }
  //gpBoxFunction->Visible = false;
  Form1->Refresh();
  int index;
  msg[2]=0;
  strcpy(title, cBoxGuest->Text.c_str() );

  //for (int n=0; n<1; n++)
  while (true)
     { index = 0;
       msg[0] = title[index];
       msg[1] = title[index+1];
       banner0.setText(msg);
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 0, 0, 0.66, msg);
       index += 2;
       msg[0] = title[index];
       msg[1] = title[index+1];
       banner0.setText(msg);
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 300, 0, 0.66, msg);
       index += 2;
       msg[0] = title[index];
       msg[1] = title[index+1];
       banner0.setText(msg);
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 600, 0, 0.66, msg);
       Application->ProcessMessages();
       banner0.setText("z");
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 150, 300, 0.7, "z");
       banner0.setText("n");
       banner0.getBanner(Form1->Canvas, 0, 0);
       banner0.rotatingShow(Form1->Canvas, 450, 300, 0.7, "n");
       Form1->Canvas->TextOut(300, 620, "FnqtBqPqTt wzII");
       Application->ProcessMessages();
       if ( !chkRepeat->Checked ) break;
     }
  return;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormCreate(TObject *Sender)
{
  cBoxGuest->ItemIndex = 0;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormActivate(TObject *Sender)
{ // setting the default font
  Form1->Canvas->Font = FontDialog1->Font;
  Image1->Canvas->Font= FontDialog1->Font;
  gpBoxFunction->Left = ClientWidth - gpBoxFunction->Width -2;
  gpBoxFunction->Top  = ClientHeight- gpBoxFunction->Height-2;
}
//---------------------------------------------------------------------------

